/*
 * LCD4x40_430.C
 *
 *  Created on: 7/dic/2013
 *
 *
 *
 * M0dulo LCD-4x40
 * E' un modulo da impiegarsi co un display LCD a 4 righe e 40 caratteri
 * Questo LCD ha due controllori uno per le prime due righe un secondo per le altre due:
 * Si comporta quindi come fossero due LCD distinti con 2 righe di 40 caratteri ciascuno.
 * I dati presenti alla porta comune si acquisiscono con E1 oppure E2
 */

/*----------------------------------------------------------------------*/

#include <msp430.h>
#include "LCD4x40_430.h"


extern char flag;
extern char LCD_data;
extern int  I2C_State;



/*
 * void dys_clk()
 *
 * Genera un inpuso di clock su ambedue i controllori del display E1 e E2
 * Il clock agisce sulle quattro righe dell'LCD
 */
void dys_clk(){   // Tutto il displey, controllori 1 e 2
    P2OUT |= BIT3 + BIT5;     	//P2.3 P2.5  a 1
    _NOP();
    _NOP();
    P2OUT &= ~BIT3 + ~BIT5;		//P2.3 P2.5  a 1
  }

/*
 * void dys_clk1()
 *
 * Genera un inpuso di clock su E1
 * Il clock1 agisce sulle righe uno e due
 */
void dys_clk1(){   // Prime due righe  Controllore 1
    P2OUT |= BIT3;		// P2.3  a 1
    _NOP();
    _NOP();
    P2OUT &= ~BIT3;		// P2.3  a 0
  }

/*
 * void dys_clk2()
 *
 * Genera un inpuso di clock su E2
 * Il clock2 agisce sulle righe tre e quattro
 */
void dys_clk2(){   // Seconde due righe controllore 2
    P2OUT |= BIT5;		// P2.5  a 1
    _NOP();
    _NOP();
    P2OUT &= ~BIT5;		// P2.5  a 1
   }


/*
 * void dys_clr(char clock)
 *
 * clock =1 = E1 le prime due righe
 * clock =2 = E2 le seconde due righe
 * clock =3 = E1 + E2 tutto il display
 */
void dys_clr(char clock){
    P2OUT &= ~BIT4;           //  RS = 0
    write_i2c(0x01,clock);	// scrive 0x01
  }


/*
 * void cursore_off(char clock)
 *
 * ammessi: clock=1=E1 oppure clock=2=E2
 * Spegne il cursore dell'LCD
 */
void cursore_off(char clock){
    P2OUT &= ~BIT4;           //  RS = 0
    write_i2c(0x0C,clock);	// scrive 0x0C
	}


/*
 * void cursor_blink(char Clock)
 *
 * ammessi: clock=1=E1 oppure clock=2=E2
 * Cursore lampeggiante
 */
void cursor_blink(char clock){
    P2OUT &= ~BIT4;           //  RS = 0
    write_i2c(0x0F,clock);	// scrive 0x0F
    P2OUT |= BIT4;				// RS = 1
	}


/*
 * void dys_string(const char stringa[],char clock)
 *
 * ammessi: clock=1=E1 oppure clock=2=E2
 * Si invia una stringa a display
 */
void dys_string(const char stringa[],char clock){
  char i,dato;
  i=0;
  do{
	    P2OUT |= BIT4;           //  RS = 1
//	    ctll = cntr;
	    dato = stringa[i];
	    if(dato<0x20||dato>0X7A) break;      //se non  un carattere valivo
	    write_i2c(dato,clock);	             // scrive il byte
	    i= i+1;
    } while(dato !=0);
  P2OUT &= ~BIT4;           //  RS = 0
}


/*
 * void dys_byte(char data,char ctrl)
 *
 * ammessi: clock=1=E1 oppure clock=2=E2
 * manda a dysplay un byte
 */
void dys_byte(char data,char clock){
    P2OUT |= BIT4;				//  RS = 1
  	write_i2c(data,clock);		// scrive data
	P2OUT &= ~BIT4;           	//  RS = 0
}

/*
 * dys_num(word dato,char clock)
 *
 * Converte il numero (max 65535) nei corrispondenti valori esadecimali
 * srivendoli nel vettore val[],poi manda la stringa a display
 * ammessi: clock=1=E1 oppure clock=2=E2
 * Scrive in val[j] i valori es:135 0x31,0x33,0x35
 */
  void dys_num(unsigned int dato,char clock)
    {
      char val[5];
	  char j;
      unsigned int a,b,n;

      n=10000;
      b=dato;

      for(j=0;j<5;j++)
        {
          a=b/n;
          val[j]=a+0x30;
          b=b-(a*n);
          n=n/10;
        }
    dys_string(val,clock);

    }


/*
 * void comand(char pos_riga,char riga,char clock)
 *
 * ammessi: clock=1=E1 oppure clock=2=E2
 * Si unvia un byte di comando per l'LCD definendo dove scrivere
 * quindi posizione sulla riga, riga, quale controllore
 * Alla posizione viene sottratto 1 perch si parte dalla posizione 1
 * incece che 0
 */
void comand(char pos_riga,char riga,char clock){
   char j;

   P2OUT &= ~BIT4;           //  RS = 0
   if(riga == 1){
	   j = pos_riga;
	   j=j+128-1;                 /* indirizzo prima cella = $80=128 */
      }
   if (riga == 2){
	   j = pos_riga;
	   j=j+192-1;                 /* indirizzo prima cella = $C0=192 */
      }
   write_i2c(j,clock);
   P2OUT |= BIT4;				// RS = 1
}

/*
 * void reset_dys()
 *
 * Il Reset del display si fa con 0x38 e tre impulsi di clock poi con
 *   0x01 0x0E  0x0C intervallati naturalmente da un impulso di clock
 */
void reset_dys(){
	   write_i2c(0x38,3);
	   __delay_cycles (1000);

	   dys_clk();
	    __delay_cycles (100);

	    dys_clk();
	     __delay_cycles (100);

	   // Ox01
	   write_i2c(0x01,3);
	    __delay_cycles (20);

	    // Ox0E
	    write_i2c(0x0E,3);
	     __delay_cycles (20);


	     // Ox0C
	     write_i2c(0x0C,3);
	      __delay_cycles (20);
	  }


/*
 *	write_i2c(char lcd_data)
 *
 *	invia  il dato all'expander, Poi con uno strobe memorizza  il
 *	dato che si visualizza sull'LCD
 * clock =1 = E1 le prime due righe
 * clock =2 = E2 le seconde due righe
 * clock =3 = E1 + E2 tutto il display
 */
void write_i2c(char lcd_data,char clock){

	I2C_State = 0;
	LCD_data = lcd_data;
	  while(flag == 0)
	  {
	    USICTL1 |= USIIFG;          // Set flag and start communication
	    LPM0;                       // CPU off, await USI interrupt
	    _NOP();                     // Used for IAR
	    __delay_cycles (2000);
	  }
  flag = 0;							//ripristino il flag

/*
 * A questo punto  presente sull'uscita del PCF8574A il dato viene ora inviato
 * un impulso di clock per memorizzare il dato su uno o entrambi i controllori
 * dell' LCD
 */

  if(clock == 1){				//prime due righe
  	dys_clk1();
  }
  else if(clock == 2){		// Seconde due righe
  	dys_clk2();
  }
  else if(clock == 3){		// tutto il display
  	dys_clk();
  }
      }













